﻿--
-- $Id: BugGrabber.lua 158 2010-10-25 10:40:10Z rabbit $
--
-- The BugSack and BugGrabber team is:
-- Current Developer: Rabbit
-- Past Developers: Rowne, Ramble, industrial, Fritti, kergoth, ckknight
-- Testers: Ramble, Sariash
--
--[[

BugGrabber, World of Warcraft addon that catches errors and formats them with a debug stack.
Copyright (C) 2008 The BugGrabber Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

]]

-----------------------------------------------------------------------
-- Global config variables

MAX_BUGGRABBER_ERRORS = 1000

-- If we get more errors than this per second, we stop all capturing until the
-- user tells us to continue.
BUGGRABBER_ERRORS_PER_SEC_BEFORE_THROTTLE = 20
BUGGRABBER_TIME_TO_RESUME = 60
BUGGRABBER_SUPPRESS_THROTTLE_CHAT = nil

-----------------------------------------------------------------------
-- Localization
local L = {
	NO_DISPLAY_1 = "|cffff4411You seem to be running !BugGrabber with no display addon to go along with it. Although !BugGrabber provides a slash command for accessing in-game errors, a display addon can help you manage these errors in a more convenient way.|r",
	NO_DISPLAY_2 = "|cffff4411The standard !BugGrabber display is called |r|cff44ff44BugSack|r|cffff4411, and can probably be found on the same site where you found !BugGrabber.|r",
	NO_DISPLAY_STOP = "|cffff4411If you don't want to be reminded about this again, please run |cff44ff44/stopnag|r|cffff4411.|r",
	STOP_NAG = "|cffff4411!BugGrabber will not nag about missing |r|cff44ff44BugSack|r|cffff4411 again until next patch.|r",
	CMD_CREATED = "An error has been detected, use /buggrabber to print it.",
	USAGE = "Usage: /buggrabber <1-%d>.",
	ERROR_INDEX = "The provided index must be a number.",
	ERROR_UNKNOWN_INDEX = "The index %d does not exist in the load error table.",
	STARTUP_ERRORS = "There were %d startup errors:",
	STARTUP_ERRORS_MANY = "There were %d startup errors, please use /buggrabber <number> to print them.",
	UNIQUE_CAPTURE = "BugGrabber captured a unique error:\n%s\n---",
	ADDON_CALL_PROTECTED = "[%s] AddOn '%s' tried to call the protected function '%s'.",
	ADDON_CALL_PROTECTED_MATCH = "^%[(.*)%] (AddOn '.*' tried to call the protected function '.*'.)$",
	ADDON_DISABLED = "|cffffff7fBugGrabber|r and |cffffff7f%s|r cannot coexist together. |cffffff7f%s|r has been disabled because of this. If you want to, you may exit out, disable |cffffff7fBugGrabber|r and reenable |cffffff7f%s|r.",
	BUGGRABBER_STOPPED = "|cffffff7fBugGrabber|r has stopped capturing errors, since it has captured more than %d errors per second. Capturing will resume in %d seconds.",
	BUGGRABBER_RESUMING = "|cffffff7fBugGrabber|r is capturing errors again.",
}
do
	local locale = GetLocale()
	if locale == "koKR" then
L["ADDON_CALL_PROTECTED"] = "[%s] \236\149\160\235\147\156\236\152\168 '%s' \235\179\180\237\152\184\235\144\156 \237\149\168\236\136\152 \237\152\184\236\182\156 '%s'."
L["ADDON_CALL_PROTECTED_MATCH"] = "^%[(.*)%] (\236\149\160\235\147\156\236\152\168 '.*' \235\179\180\237\152\184\235\144\156 \237\149\168\236\136\152 \237\152\184\236\182\156 '.*'.)$"
L["ADDON_DISABLED"] = "|cffffff7fBugGrabber|r\236\153\128 |cffffff7f%s|r\235\138\148 \237\149\168\234\187\152 \234\179\181\236\161\180\237\149\160 \236\136\152 \236\151\134\236\138\181\235\139\136\235\139\164. |cffffff7f%s|r\236\151\144 \236\157\152\237\149\180 \236\164\145\236\167\128\235\144\152\236\151\136\236\138\181\235\139\136\235\139\164. \235\167\140\236\149\189 \235\139\185\236\139\160\236\157\180 \236\155\144\237\149\152\235\169\180, \236\160\145\236\134\141\236\157\132 \236\162\133\235\163\140\237\149\156 \237\155\132, |cffffff7fBugGrabber|r\235\165\188 \236\164\145\236\167\128\237\149\152\234\179\160 |cffffff7f%s|r\235\165\188 \236\158\172\237\153\156\236\132\177\237\149\152\236\132\184\236\154\148."
L["BUGGRABBER_RESUMING"] = "|cffffff7fBugGrabber|r\234\176\128 \235\139\164\236\139\156 \236\152\164\235\165\152\235\165\188 \236\186\161\236\179\144\237\149\169\235\139\136\235\139\164."
L["BUGGRABBER_STOPPED"] = "\236\157\180\234\178\131\236\157\128 \236\180\136\235\139\185 %d\234\176\156 \236\157\180\236\131\129\236\157\152 \236\152\164\235\165\152\235\165\188 \235\176\156\234\178\172\237\149\152\236\152\128\234\184\176\236\151\144 |cffffff7fBugGrabber|r\236\157\152 \236\152\164\235\165\152 \236\186\161\236\179\144\234\176\128 \236\164\145\236\167\128\235\144\152\236\151\136\236\156\188\235\169\176, \236\186\161\236\179\144\235\138\148 %d\236\180\136 \237\155\132 \236\158\172\234\176\156\235\144\169\235\139\136\235\139\164."
L["CMD_CREATED"] = "\236\152\164\235\165\152\234\176\128 \235\176\156\234\178\172\235\144\152\236\151\136\236\156\188\235\169\176, /buggrabber\235\165\188 \236\130\172\236\154\169\237\149\152\236\151\172 \236\182\156\235\160\165\237\149\160 \236\136\152 \236\158\136\236\138\181\235\139\136\235\139\164."
L["ERROR_INDEX"] = "\236\131\137\236\157\184\234\176\146\236\157\128 \236\136\171\236\158\144\236\157\180\236\150\180\236\149\188 \237\149\169\235\139\136\235\139\164."
L["ERROR_UNKNOWN_INDEX"] = "\235\182\136\235\159\172\236\152\168 \236\152\164\235\165\152 \235\170\169\235\161\157\236\151\144 %d\235\178\136\236\167\184 \236\152\164\235\165\152\235\138\148 \236\161\180\236\158\172\237\149\152\236\167\128 \236\149\138\236\138\181\235\139\136\235\139\164."
L["STARTUP_ERRORS"] = "%d\234\176\156\236\157\152 \236\139\156\236\158\145\236\139\156 \236\152\164\235\165\152\234\176\128 \235\176\156\236\131\157:"
L["STARTUP_ERRORS_MANY"] = "%d\234\176\156\236\157\152 \236\139\156\236\158\145\236\139\156 \236\152\164\235\165\152\234\176\128 \235\176\156\236\131\157\237\150\136\236\138\181\235\139\136\235\139\164, /buggrabber <\236\136\171\236\158\144>\235\165\188 \236\130\172\236\154\169\237\149\180\236\132\156 \237\149\180\235\139\185 \236\152\164\235\165\152\235\165\188 \235\179\188 \236\136\152 \236\158\136\236\138\181\235\139\136\235\139\164."
L["UNIQUE_CAPTURE"] = "BugGrabber\234\176\128 \235\176\156\234\178\172\237\149\156 \236\152\164\235\165\152:\
%s\
---"
L["USAGE"] = "\236\130\172\236\154\169\235\178\149: /buggrabber <1-%d>."

	elseif locale == "deDE" then
L["ADDON_CALL_PROTECTED"] = "[%s] AddOn '%s' hat versucht die gesch\195\188tzte Funktion '%s' aufzurufen."
L["ADDON_CALL_PROTECTED_MATCH"] = "^%[(.*)%] (AddOn '.*' hat versucht die gesch\195\188tzte Funktion '.*' aufzurufen.)$"
L["ADDON_DISABLED"] = "|cffffff7fBugGrabber|r und |cffffff7f%s|r k\195\182nnen nicht zusammen laufen, |cffffff7f%s|r wurde deshalb deaktiviert. Du kannst jetzt WoW schlie\195\159en, |cffffff7fBugGrabber|r deaktivieren und |cffffff7f%s|r erneut aktivieren."
L["BUGGRABBER_RESUMING"] = "|cffffff7fBugGrabber|r zeichnet nun wieder Fehler auf."
L["BUGGRABBER_STOPPED"] = "|cffffff7fBugGrabber|r hat die Aufzeichnung von Fehlern gestoppt, weil mehr als %d Fehler pro Sekunde erzeugt wurden. Die Aufzeichnung wird in %d Sekunden fortgesetzt."
L["CMD_CREATED"] = "Ein Fehler wurde entdeckt, benutze /buggrabber um ihn anzuzeigen."
L["ERROR_INDEX"] = "Der zur Verf\195\188gung gestellte Index mu\195\159 eine Zahl sein."
L["ERROR_UNKNOWN_INDEX"] = "Der Index %d existiert nicht in der geladenen Fehlerliste."
L["STARTUP_ERRORS"] = "Es gab %d Fehler beim Start:"
L["STARTUP_ERRORS_MANY"] = "Es gab %d Fehler beim Start, verwende bitte /buggrabber <Anzahl> um sie aufzulisten."
L["UNIQUE_CAPTURE"] = "BugGrabber hat einen einzigartigen Fehler aufgezeichnet:\
%s\
---"
L["USAGE"] = "Benutzung: /buggrabber <1-%d>."

	elseif locale == "esES" then
L["ADDON_CALL_PROTECTED"] = "[%s] El accesorio '%s' ha intentado llamar a la funci\195\179n protegida '%s'."
L["ADDON_CALL_PROTECTED_MATCH"] = "^%[(.*)%] (El accesorio '.*' ha intentado llamar a la funci\195\179n protegida '.*'.)$"
L["ADDON_DISABLED"] = "|cffffff7fBugGrabber|r y |cffffff7f%s|r no pueden coexistir juntos. |cffffff7f%s|r ha sido desactivado por este motivo. Si lo deseas, puedes salir, desactivar |cffffff7fBugGrabber|r y reactivar |cffffff7f%s|r."
L["BUGGRABBER_RESUMING"] = "|cffffff7fBugGrabber|r est\195\161 capturando errores de nuevo."
L["BUGGRABBER_STOPPED"] = "|cffffff7fBugGrabber|r ha detenido la captuta de errores, ya que ha capturado m\195\161s de %d errores por segundo. La captura se reanudar\195\161 en %d segundos."
L["CMD_CREATED"] = "Un error ha sido detectado, utiliza /buggrabber para imprimirlo."
L["ERROR_INDEX"] = "El \195\173ndice introducido debe ser un n\195\186mero."
L["ERROR_UNKNOWN_INDEX"] = "El \195\173ndice %d no existe en la tabla de errores de carga."
L["STARTUP_ERRORS"] = "Ha habido %d errores de arranque:"
L["STARTUP_ERRORS_MANY"] = "Ha habido %d errores de arranque, por favor, usa /buggrabber <number> para listarlos."
L["UNIQUE_CAPTURE"] = "BugGrabber ha capturado un \195\186nico error:\
%s\
---"
L["USAGE"] = "Uso: /buggrabber <1-%d>."

	elseif locale == "zhTW" then
L["ADDON_CALL_PROTECTED"] = "[%s] \230\143\146\228\187\182 '%s' \229\152\151\232\169\166\232\170\191\231\148\168\228\191\157\232\173\183\229\138\159\232\131\189 '%s'\227\128\130"
L["ADDON_CALL_PROTECTED_MATCH"] = "^%[(.*)%] (\230\143\146\228\187\182 '.*' \229\152\151\232\169\166\232\170\191\231\148\168\228\191\157\232\173\183\229\138\159\232\131\189 '.*'.)$"
L["ADDON_DISABLED"] = "|cffffff7fBugGrabber|r \229\146\140 |cffffff7f%s|r \228\184\141\232\131\189\229\133\177\229\173\152\227\128\130|cffffff7f%s|r \229\183\178\229\129\156\231\148\168\227\128\130\229\143\175\229\156\168\230\143\146\228\187\182\228\187\139\233\157\162\229\129\156\231\148\168 |cffffff7fBugGrabber|r\239\188\140\229\134\141\231\148\168 |cffffff7f%s|r\227\128\130"
L["BUGGRABBER_RESUMING"] = "|cffffff7fBugGrabber|r \229\183\178\233\135\141\230\150\176\233\150\139\229\167\139\227\128\130"
L["BUGGRABBER_STOPPED"] = "|cffffff7fBugGrabber|r \231\143\190\230\173\163\230\154\171\229\129\156\239\188\140\229\155\160\231\130\186\230\175\143\231\167\146\230\141\149\230\141\137\229\136\176\232\182\133\233\129\142%d\229\128\139\233\140\175\232\170\164\227\128\130\229\174\131\230\156\131\229\156\168%d\231\167\146\229\190\140\233\135\141\230\150\176\233\150\139\229\167\139\227\128\130"
L["CMD_CREATED"] = "\231\153\188\231\143\190\233\140\175\232\170\164\239\188\140\231\148\168 /buggrabber \229\136\151\229\135\186\233\128\153\233\140\175\232\170\164\227\128\130"
L["ERROR_INDEX"] = "\230\143\144\228\190\155\231\154\132\231\180\162\229\188\149\229\128\188\229\191\133\233\160\136\230\152\175\230\149\184\229\173\151\227\128\130"
L["ERROR_UNKNOWN_INDEX"] = "\230\143\144\228\190\155\231\154\132\231\180\162\229\188\149\229\128\188\227\128\140%d\227\128\141\228\184\141\230\152\175\230\173\163\231\162\186\231\154\132\227\128\130"
L["STARTUP_ERRORS"] = "\229\156\168\229\149\159\229\139\149\230\153\130\231\153\188\231\148\159%d\229\128\139\233\140\175\232\170\164\239\188\154"
L["STARTUP_ERRORS_MANY"] = "\229\156\168\229\149\159\229\139\149\230\153\130\231\153\188\231\148\159%d\229\128\139\233\140\175\232\170\164\239\188\140\232\171\139\228\189\191\231\148\168 /buggrabber <\231\180\162\229\188\149\229\128\188> \228\190\134\229\136\151\229\135\186\227\128\130"
L["UNIQUE_CAPTURE"] = "\230\141\149\230\141\137\229\136\176\230\150\176\231\154\132\233\140\175\232\170\164\239\188\154\
%s\
---"
L["USAGE"] = "\231\148\168\230\179\149\239\188\154/buggrabber <1-%d>\227\128\130"

	elseif locale == "zhCN" then
L["ADDON_CALL_PROTECTED"] = "[%s] \230\143\146\228\187\182 '%s' \229\176\157\232\175\149\232\176\131\231\148\168\228\191\157\230\138\164\229\138\159\232\131\189 '%s'\227\128\130"
L["ADDON_CALL_PROTECTED_MATCH"] = "^%[(.*)%] (\230\143\146\228\187\182 '.*' \229\176\157\232\175\149\232\176\131\231\148\168\228\191\157\230\138\164\229\138\159\232\131\189 '.*'.)$"
L["ADDON_DISABLED"] = "|cffffff7fBugGrabber|r \229\146\140 |cffffff7f%s|r \228\184\141\232\131\189\229\133\177\229\173\152\227\128\130|cffffff7f%s|r \229\183\178\229\129\156\231\148\168\227\128\130\229\143\175\229\156\168\230\143\146\228\187\182\231\149\140\233\157\162\229\129\156\231\148\168 |cffffff7fBugGrabber|r \229\134\141\231\148\168 |cffffff7f%s|r\227\128\130"
L["BUGGRABBER_RESUMING"] = "|cffffff7fBugGrabber|r \229\183\178\233\135\141\230\150\176\229\188\128\229\167\139\227\128\130"
L["BUGGRABBER_STOPPED"] = "|cffffff7fBugGrabber|r \231\142\176\230\173\163\230\154\130\229\129\156\239\188\140\229\155\160\228\184\186\230\175\143\231\167\146\230\141\149\230\141\137\229\136\176\232\182\133\232\191\135%d\228\184\170\233\148\153\232\175\175\227\128\130\229\174\131\228\188\154\229\156\168%d\231\167\146\229\144\142\233\135\141\230\150\176\229\188\128\229\167\139\227\128\130"
L["CMD_CREATED"] = "\229\143\145\231\142\176\228\184\128\228\184\170\233\148\153\232\175\175\239\188\140\231\148\168 /buggrabber \229\136\151\229\135\186\232\191\153\228\184\170\233\148\153\232\175\175\227\128\130"
L["ERROR_INDEX"] = "\230\143\144\228\190\155\231\154\132\231\180\162\229\188\149\229\128\188\229\191\133\233\161\187\230\152\175\230\149\176\229\173\151\227\128\130"
L["ERROR_UNKNOWN_INDEX"] = "\230\143\144\228\190\155\231\154\132\231\180\162\229\188\149\229\128\188\227\128\140%d\227\128\141\228\184\141\230\152\175\230\173\163\231\161\174\231\154\132\227\128\130"
L["STARTUP_ERRORS"] = "\229\156\168\229\144\175\229\138\168\230\151\182\229\143\145\231\148\159%d\228\184\170\233\148\153\232\175\175\239\188\154"
L["STARTUP_ERRORS_MANY"] = "\229\156\168\229\144\175\229\138\168\230\151\182\229\143\145\231\148\159%d\228\184\170\233\148\153\232\175\175\239\188\140\232\175\183\228\189\191\231\148\168 /buggrabber <\231\180\162\229\188\149\229\128\188>\230\157\165\229\136\151\229\135\186\227\128\130"
L["UNIQUE_CAPTURE"] = "BugGrabber \230\141\149\230\141\137\229\136\176\230\150\176\231\154\132\233\148\153\232\175\175\239\188\154\
%s\
---"
L["USAGE"] = "\231\148\168\230\179\149\239\188\154/buggrabber <1-%d>\227\128\130"

	elseif locale == "ruRU" then
L["ADDON_DISABLED"] = "|cffffff7fBugGrabber|r \208\184 |cffffff7f%s|r \208\189\208\181 \208\188\208\190\208\182\208\181\209\130 \209\129\209\131\209\137\208\181\209\129\209\130\208\178\208\190\208\178\208\176\209\130\209\140 \208\178\208\188\208\181\209\129\209\130\208\181, |cffffff7f%s|r \208\177\209\139\208\187 \208\189\208\181\208\184\209\129\208\191\209\128\208\176\208\178\208\189\209\139\208\185. \208\181\209\129\208\187\208\184 \209\133\208\190\209\130\208\184\209\130\208\181 \208\178\209\139\208\180\208\184\209\130\208\181 \208\184\208\183 WoW nun \208\190\209\130\208\186\208\187\209\142\209\135\208\184\209\130\208\181 \208\189\208\181\208\184\209\129\208\191\209\128\208\176\208\178\208\189\209\139\208\185 \208\176\208\180\208\180\208\190\208\189, |cffffff7fBugGrabber|r \208\191\208\190\208\178\209\130\208\190\209\128\208\189\208\190 \208\183\208\176\208\191\209\131\209\129\209\130\208\184\208\187 \208\176\208\180\208\180\208\190\208\189|cffffff7f%s|r."
L["BUGGRABBER_RESUMING"] = "|cffffff7fBugGrabber|r \208\183\208\176\209\133\208\178\208\176\209\130\208\184\208\187 \208\190\209\136\208\184\208\177\208\186\208\184 \209\129\208\189\208\190\208\178\208\176."
L["BUGGRABBER_STOPPED"] = "|cffffff7fBugGrabber|r \208\191\209\128\208\181\208\186\209\128\208\176\209\130\208\184\208\187 \208\183\208\176\209\133\208\178\208\176\209\130\209\139\209\130\208\178\208\176\209\130\209\140 \208\190\209\136\208\184\208\177\208\186\208\184, \209\130\208\176\208\186 \208\186\208\176\208\186 \208\190\208\189 \208\183\208\176\209\133\208\178\208\176\209\130\208\184\208\187 \208\177\208\190\208\187\209\140\209\136\208\181 %d \208\190\209\136\208\184\208\177\208\190\208\186  \208\178 \209\129\208\181\208\186\209\131\208\189\208\180\209\131. \208\151\208\176\209\133\208\178\208\176\209\130 \208\178\208\190\208\183\208\190\208\177\208\189\208\190\208\178\208\184\209\130\209\140\209\129\209\143 \208\178 \209\130\208\181\209\135\208\181\208\189\208\184\208\184 %d \208\161\208\181\208\186\209\131\208\189\208\180."
L["CMD_CREATED"] = "\208\158\209\136\208\184\208\177\208\186\208\176 \208\177\209\139\208\187\208\176 \208\190\208\177\208\189\208\176\209\128\209\131\208\182\208\181\208\189\208\176, \209\129\208\191\208\190\208\187\209\140\208\183\209\131\208\185\209\130\208\181 /buggrabber \209\135\209\130\208\190\208\177\209\139 \208\189\208\176\208\191\208\181\209\135\208\176\209\130\208\176\209\130\209\140 \208\181\208\181."
L["ERROR_INDEX"] = "\208\159\209\128\208\181\208\180\208\190\209\129\209\130\208\176\208\178\208\187\208\181\208\189\208\189\209\139\208\185 \208\184\208\189\208\180\208\181\208\186\209\129 \208\180\208\190\208\187\208\182\208\181\208\189 \208\177\209\139\209\130\209\140 \209\135\208\184\209\129\208\187\208\190\208\188"
L["ERROR_UNKNOWN_INDEX"] = "\208\152\208\189\208\180\208\181\208\186\209\129  %d  \208\189\208\181 \209\129\209\131\209\137\208\181\209\129\209\130\208\178\209\131\208\181\209\130 \208\178 \208\183\208\176\208\179\209\128\209\131\208\182\208\181\208\189\208\189\208\190\208\185 \209\130\208\176\208\177\208\187\208\184\209\134\208\181 \208\190\209\136\208\184\208\177\208\190\208\186."
L["STARTUP_ERRORS"] = "\208\177\209\139\208\187\208\184 %d \208\158\209\136\208\184\208\177\208\186\208\176\208\188\208\184 \208\161\209\130\208\176\209\128\209\130\208\176:"
L["STARTUP_ERRORS_MANY"] = "\208\145\209\139\208\187\208\184 %d \208\158\209\136\208\184\208\177\208\186\208\176\208\188\208\184 \208\151\208\176\208\191\209\131\209\129\208\186\208\176, \208\184\209\129\208\191\208\190\208\187\209\140\208\183\209\131\208\185\209\130\208\181 /buggrabber <\208\189\208\190\208\188\208\181\209\128> \209\135\209\130\208\190\208\177\209\139 \208\184\209\133 \208\189\208\176\208\191\208\181\209\135\208\176\209\130\208\176\209\130\209\140."
L["UNIQUE_CAPTURE"] = "BugGrabber \208\183\208\176\209\133\208\178\208\176\209\130\208\184\208\187 \209\131\208\189\208\184\208\186\208\176\208\187\209\140\208\189\209\131\209\142 \208\190\209\136\208\184\208\177\208\186\209\131:\
%s\
---"
L["USAGE"] = "\208\152\209\129\208\191\208\190\208\187\209\140\208\183\208\190\208\178\208\176\208\189\208\184\208\181: /buggrabber <1-%d>."

	elseif locale == "frFR" then
L["ADDON_CALL_PROTECTED"] = "[%s] L'AddOn '%s' a tent\195\169 d'appeler la fonction prot\195\169g\195\169e '%s'."
L["ADDON_CALL_PROTECTED_MATCH"] = "^%[(.*)%] (L'AddOn '.*' a tent\195\169 d'appeler la fonction prot\195\169g\195\169e '.*'.)$"
L["ADDON_DISABLED"] = "|cffffff7fBugGrabber|r et |cffffff7f%s|r ne peuvent pas \195\170tre lanc\195\169s en m\195\170me temps. |cffffff7f%s|r a \195\169t\195\169 d\195\169sactiv\195\169. Si vous le souhaitez, vous pouvez vous d\195\169connecter, d\195\169sactiver |cffffff7fBugGrabber|r et r\195\169activer |cffffff7f%s|r."
L["BUGGRABBER_RESUMING"] = "|cffffff7fBugGrabber|r capture les erreurs \195\160 nouveau."
L["BUGGRABBER_STOPPED"] = "|cffffff7fBugGrabber|r a cess\195\169 de capturer des erreurs, car plus de %d erreurs ont \195\169t\195\169 captur\195\169es par seconde. La capture sera reprise dans %d secondes."
L["CMD_CREATED"] = "Une erreur a \195\169t\195\169 d\195\169tect\195\169e, tapez /buggrabber pour l'afficher."
L["ERROR_INDEX"] = "L'\195\169l\195\169ment donn\195\169 doit \195\170tre un nombre."
L["ERROR_UNKNOWN_INDEX"] = "L'\195\169l\195\169ment %d n'existe pas dans la table d'erreurs charg\195\169e."
L["STARTUP_ERRORS"] = "Il y a eu %d erreurs au d\195\169marrage:"
L["STARTUP_ERRORS_MANY"] = "Il y a eu %d erreurs au d\195\169marrage, merci d'utiliser /buggrabber <nombre> pour les afficher."
L["USAGE"] = "Utilisation: /buggrabber <1-%d>."

	elseif locale == "esMX" then

	end
end
-----------------------------------------------------------------------

local BugGrabber = {}
local frame = CreateFrame("Frame")

local real_seterrorhandler = seterrorhandler

-- Fetched from X-BugGrabber-Display in the TOC of a display addon.
-- Should implement :FormatError(errorTable).
local displayObjectName = nil

local totalElapsed = 0
local errorsSinceLastReset = 0
local paused = nil
local looping = false

local stateErrorDatabase = {}
local slashCmdCreated = nil
local slashCmdErrorList = {}

local isBugGrabbedRegistered = false
local callbacks = nil
local function setupCallbacks()
	if not callbacks and LibStub and LibStub("CallbackHandler-1.0", true) then
		callbacks = LibStub("CallbackHandler-1.0"):New(BugGrabber)
		function callbacks:OnUsed(target, eventname)
			if eventname == "BugGrabber_BugGrabbed" then isBugGrabbedRegistered = true end
		end
		function callbacks:OnUnused(target, eventname)
			if eventname == "BugGrabber_BugGrabbed" then isBugGrabbedRegistered = false end
		end
	end
end
local function triggerEvent(...)
	if not callbacks then setupCallbacks() end
	if callbacks then callbacks:Fire(...) end
end

local function slashHandler(index)
	if not index or tostring(index) == "" then
		print(L.USAGE:format(#slashCmdErrorList))
		return
	end
	if not tonumber(index) then
		print(L.ERROR_INDEX)
		return
	end
	index = tonumber(index)
	if not slashCmdErrorList[index] then
		print(L.ERROR_UNKNOWN_INDEX:format(index))
		return
	end
	local err = slashCmdErrorList[index]
	if type(err) ~= "table" or (type(err.message) ~= "string" and type(err.message) ~= "table") then return end
	local found = nil
	if displayObjectName and _G[displayObjectName] then
		local display = _G[displayObjectName]
		if type(display) == "table" and type(display.FormatError) == "function" then
			found = true
			print(tostring(index) .. ". " .. display:FormatError(err))
		end
	end
	if not found then
		local m = err.message
		if type(m) == "table" then
			m = table.concat(m, "")
		end
		print(tostring(index) .. ". " .. m)
	end
end

local function createSlashCmd()
	if slashCmdCreated then return end
	local name = "BUGGRABBERCMD"
	local counter = 0
	repeat
		name = "BUGGRABBERCMD"..tostring(counter)
		counter = counter + 1
	until not _G.SlashCmdList[name] and not _G["SLASH_"..name.."1"]
	_G.SlashCmdList[name] = slashHandler
	_G["SLASH_"..name.."1"] = "/buggrabber"

	slashCmdCreated = true
	if not isBugGrabbedRegistered then
		print(L.CMD_CREATED)
	end
end

local function saveError(message, errorType)
	-- Start with the date, time and session
	local oe = {}
	oe.message = message .. "\n  ---"
	oe.session = BugGrabberDB.session
	oe.time = date("%Y/%m/%d %H:%M:%S")
	oe.type = errorType
	oe.counter = 1

	-- WoW crashes when strings > 983 characters are stored in the
	-- SavedVariables file, so make sure we don't exceed that limit.
	-- For lack of a better thing to do, just truncate the error :-/
	if oe.message:len() > 980 then
		local m = oe.message
		oe.message = {}
		local maxChunks, chunks = 5, 0
		while m:len() > 980 and chunks <= maxChunks do
			local q
			q, m = m:sub(1, 980), m:sub(981)
			oe.message[#oe.message + 1] = q
			chunks = chunks + 1
		end
		if m:len() > 980 then m = m:sub(1, 980) end
		oe.message[#oe.message + 1] = m
	end

	-- Insert the error into the correct database if it's not there already.
	-- If it is, just increment the counter.
	local found = false
	local db = BugGrabber:GetDB()
	local oe_message = oe.message
	if type(oe_message) == "table" then
		oe_message = oe_message[1]
	end
	for i, err in next, db do
		local err_message = err.message
		if type(err_message) == "table" then
			err_message = err_message[1]
		end
		if err_message == oe_message and err.session == oe.session then
			-- This error already exists in the current session, just increment
			-- the counter on it.
			if type(err.counter) ~= "number" then
				err.counter = 1
			end
			err.counter = err.counter + 1

			oe = nil
			oe = err

			found = true
			break
		end
	end

	-- If the error was not found in the current session, append it to the
	-- database.
	if not found then
		BugGrabber:StoreError(oe)
	end

	-- Trigger event.
	if not looping then
		local e = "BugGrabber_" .. (errorType == "event" and "Event" or "Bug") .. "Grabbed" .. (found and "Again" or "")
		triggerEvent(e, oe)

		if not found then
			slashCmdErrorList[#slashCmdErrorList + 1] = oe
			createSlashCmd()
		end
	end
end

function BugGrabber:StoreError(errorObject)
	local db = BugGrabber:GetDB()
	db[#db + 1] = errorObject

	-- Save only the last <limit> errors (otherwise the SV gets too big)
	if #db > BugGrabberDB.limit then
		table.remove(db, 1)
	end
end

local function scan(o)
	local version, revision = nil, nil
	for k, v in pairs(o) do
		if type(k) == "string" then
			local low = k:lower()
			if not version and (low == "version" or low:find("version")) and (type(v) == "string" or type(v) == "number") then
				version = v
			elseif not revision and (low == "rev" or low:find("revision")) and (type(v) == "string" or type(v) == "number")  then
				revision = v
			end
		end
		if version and revision then break end
	end
	return version, revision
end

-- Error handler
local function grabError(err)
	if paused then return end
	err = tostring(err)

	-- Get the full backtrace
	local real =
		err:find("^.-([^\\]+\\)([^\\]-)(:%d+):(.*)$") or
		err:find("^%[string \".-([^\\]+\\)([^\\]-)\"%](:%d+):(.*)$") or
		err:find("^%[string (\".-\")%](:%d+):(.*)$") or err:find("^%[C%]:(.*)$")

	err = err .. "\n" .. debugstack(real and 4 or 3)
	local errorType = "error"

	-- Normalize the full paths into last directory component and filename.
	local errmsg = ""
	looping = false

	for trace in err:gmatch("(.-)\n") do
		local match, found, path, file, line, msg, _
		found = false

		-- First detect an endless loop so as to abort it below
		if trace:find("BugGrabber") then
			looping = true
		end

		-- "path\to\file-2.0.lua:linenum:message" -- library
		if not found then
			match, _, path, file, line, msg = trace:find("^.-([^\\]+\\)([^\\]-%-%d+%.%d+%.lua)(:%d+):(.*)$")
			local addon = trace:match("^.-[A%.][d%.][d%.][Oo]ns\\([^\\]-)\\")
			if match then
				if LibStub then
					local major = file:gsub("%.lua$", "")
					local lib, minor = LibStub(major, true)
					path = major .. "-" .. (minor or "?")
					if addon then
						file = " (" .. addon .. ")"
					else
						file = ""
					end
				end
				found = true
			end
		end
		
		-- "Interface\AddOns\path\to\file.lua:linenum:message"
		if not found then
			match, _, path, file, line, msg = trace:find("^.-[A%.][d%.][d%.][Oo]ns\\(.*)([^\\]-)(:%d+):(.*)$")
			if match then
				found = true
				local addon = path:gsub("\\.*$", "")
				local addonObject = _G[addon]
				if not addonObject then
					addonObject = _G[addon:match("^[^_]+_(.*)$")]
				end
				local version, revision = nil, nil
				if LibStub and LibStub(addon, true) then
					local _, r = LibStub(addon, true)
					version = r
				end
				if type(addonObject) == "table" then
					local v, r = scan(addonObject)
					if v then version = v end
					if r then revision = r end
				end
				local objectName = addon:upper()
				if not version then version = _G[objectName .. "_VERSION"] end
				if not revision then revision = _G[objectName .. "_REVISION"] or _G[objectName .. "_REV"] end
				if not version then version = GetAddOnMetadata(addon, "Version") end
				if not version and revision then version = revision
				elseif type(version) == "string" and revision and not version:find(revision) then
					version = version .. "." .. revision
				end
				if not version then version = GetAddOnMetadata(addon, "X-Curse-Packaged-Version") end
				if version then
					path = addon .. "-" .. version .. path:gsub("^[^\\]*", "")
				end
			end
		end
		
		-- "path\to\file.lua:linenum:message"
		if not found then
			match, _, path, file, line, msg = trace:find("^.-([^\\]+\\)([^\\])(:%d+):(.*)$")
			if match then
				found = true
			end
		end

		-- "[string \"path\\to\\file.lua:<foo>\":linenum:message"
		if not found then
			match, _, path, file, line, msg = trace:find("^%[string \".-([^\\]+\\)([^\\]-)\"%](:%d+):(.*)$")
			if match then
				found = true
			end
		end

		-- "[string \"FOO\":linenum:message"
		if not found then
			match, _, file, line, msg = trace:find("^%[string (\".-\")%](:%d+):(.*)$")
			if match then
				found = true
				path = "<string>:"
			end
		end

		-- "[C]:message"
		if not found then
			match, _, msg = trace:find("^%[C%]:(.*)$")
			if match then
				found = true
				path = "<in C code>"
				file = ""
				line = ""
			end
		end

		-- ADDON_ACTION_BLOCKED
		if not found then
			match, _, file, msg = trace:find(ADDON_CALL_PROTECTED_MATCH)
			if match then
				found = true
				path = "<event>"
				file = "ADDON_ACTION_BLOCKED"
				line = ""
				errorType = "event"
			end
		end

		-- Anything else
		if not found then
			path = trace--"<unknown>"
			file = ""
			line = ""
			msg = line
		end

		-- Add it to the formatted error
		errmsg = errmsg .. path .. file .. line .. ":" .. msg .. "\n"
	end

	errorsSinceLastReset = errorsSinceLastReset + 1

	local locals = debuglocals(real and 4 or 3)
	if locals then
		errmsg = errmsg .. "\nLocals:|r\n" .. locals
	end

	-- Store the error
	saveError(errmsg, errorType)
end

local function onUpdateFunc(self, elapsed)
	totalElapsed = totalElapsed + elapsed
	if totalElapsed > 1 then
		if not paused then
			-- Seems like we're getting more errors/sec than we want.
			if errorsSinceLastReset > BUGGRABBER_ERRORS_PER_SEC_BEFORE_THROTTLE then
				BugGrabber:Pause()
			end
			errorsSinceLastReset = 0
			totalElapsed = 0
		elseif totalElapsed > BUGGRABBER_TIME_TO_RESUME and paused then
			BugGrabber:Resume()
		end
	end
end

function BugGrabber:Reset()
	stateErrorDatabase = {}
	BugGrabberDB.errors = {}
end

-- Determine the proper DB and return it
function BugGrabber:GetDB()
	return BugGrabberDB.save and BugGrabberDB.errors or stateErrorDatabase
end

function BugGrabber:GetSessionId()
	return BugGrabberDB.session
end

-- Simple setters/getters for settings, meant to be accessed by BugSack
function BugGrabber:GetSave()
	return BugGrabberDB.save
end

function BugGrabber:ToggleSave()
	BugGrabberDB.save = not BugGrabberDB.save
	if BugGrabberDB.save then
		BugGrabberDB.errors = stateErrorDatabase
		stateErrorDatabase = {}
	else
		stateErrorDatabase = BugGrabberDB.errors
		BugGrabberDB.errors = {}
	end
end

function BugGrabber:GetLimit()
	return BugGrabberDB.limit
end

function BugGrabber:SetLimit(l)
	if type(l) ~= "number" or l < 10 or l > MAX_BUGGRABBER_ERRORS then
		return
	end

	BugGrabberDB.limit = math.floor(l)
	local db = self:GetDB()
	while #db > l do
		table.remove(db, 1)
	end
end

function BugGrabber:IsThrottling()
	return BugGrabberDB.throttle
end

function BugGrabber:UseThrottling(flag)
	BugGrabberDB.throttle = flag and true or false
	if flag and not frame:GetScript("OnUpdate") then
		frame:SetScript("OnUpdate", onUpdateFunc)
	elseif not flag then
		frame:SetScript("OnUpdate", nil)
	end
end

function BugGrabber:RegisterAddonActionEvents()
	frame:RegisterEvent("ADDON_ACTION_BLOCKED")
	frame:RegisterEvent("ADDON_ACTION_FORBIDDEN")
	triggerEvent("BugGrabber_AddonActionEventsRegistered")
end

function BugGrabber:UnregisterAddonActionEvents()
	frame:UnregisterEvent("ADDON_ACTION_BLOCKED")
	frame:UnregisterEvent("ADDON_ACTION_FORBIDDEN")
	triggerEvent("BugGrabber_AddonActionEventsUnregistered")
end

function BugGrabber:IsPaused()
	return paused
end

function BugGrabber:Pause()
	if paused then return end

	if not BUGGRABBER_SUPPRESS_THROTTLE_CHAT then
		print(L.BUGGRABBER_STOPPED:format(BUGGRABBER_ERRORS_PER_SEC_BEFORE_THROTTLE, BUGGRABBER_TIME_TO_RESUME))
	end
	self:UnregisterAddonActionEvents()
	paused = true
	triggerEvent("BugGrabber_CapturePaused")
end

function BugGrabber:Resume()
	if not paused then return end

	if not BUGGRABBER_SUPPRESS_THROTTLE_CHAT then
		print(L.BUGGRABBER_RESUMING)
	end
	self:RegisterAddonActionEvents()
	paused = nil
	triggerEvent("BugGrabber_CaptureResumed")
	totalElapsed = 0
end

local function createSwatter()
	-- Need this so Stubby will feed us errors instead of just
	-- dumping them to the chat frame.
	_G.Swatter = {
		IsEnabled = function() return true end,
		OnError = function(msg, frame, stack, etype, ...)
			grabError(tostring(msg) .. tostring(stack))
		end,
	}
end

local function addonLoaded(addon)
	if addon == "!BugGrabber" then
		real_seterrorhandler(grabError)

		-- Persist defaults and make sure we have sane SavedVariables
		if type(BugGrabberDB) ~= "table" then BugGrabberDB = {} end
		local sv = BugGrabberDB
		if type(sv.session) ~= "number" then sv.session = 0 end
		if type(sv.errors) ~= "table" then sv.errors = {} end
		if type(sv.limit) ~= "number" then sv.limit = 50 end
		if type(sv.save) ~= "boolean" then sv.save = true end
		if type(sv.throttle) ~= "boolean" then sv.throttle = true end

		-- From now on we can persist errors. Create a new session.
		sv.session = sv.session + 1

		-- Determine the correct database
		local db = BugGrabber:GetDB()
		-- Cut down on the nr of errors if it is over the limit
		while #db > sv.limit do
			table.remove(db, 1)
		end
		if sv.throttle then
			frame:SetScript("OnUpdate", onUpdateFunc)
		end

		local hasDisplay = nil
		for i = 1, GetNumAddOns() do
			local meta = GetAddOnMetadata(i, "X-BugGrabber-Display")
			if meta then
				displayObjectName = meta
				hasDisplay = true
				break
			end
		end

		local currentInterface = select(4, GetBuildInfo())
		if type(currentInterface) ~= "number" then currentInterface = 0 end
		if not hasDisplay and (not sv.stopnag or sv.stopnag < currentInterface) then
			print(L.NO_DISPLAY_1)
			print(L.NO_DISPLAY_2)
			print(L.NO_DISPLAY_STOP)
			_G.SlashCmdList.BugGrabberStopNag = function()
				print(L.STOP_NAG)
				sv.stopnag = currentInterface
			end
			_G.SLASH_BugGrabberStopNag1 = "/stopnag"
		end
	elseif (addon == "!Swatter" or (type(SwatterData) == "table" and SwatterData.enabled)) and Swatter then
		print(L.ADDON_DISABLED:gsub("%%s", "Swatter"))
		DisableAddOn("!Swatter")
		SwatterData.enabled = nil
		real_seterrorhandler(grabError)
		SlashCmdList.SWATTER = nil
		SLASH_SWATTER1, SLASH_SWATTER2 = nil, nil
		for k, v in pairs(Swatter) do
			if type(v) == "table" and v.UnregisterEvent then
				v:UnregisterEvent("ADDON_ACTION_FORBIDDEN")
				v:UnregisterEvent("ADDON_ACTION_BLOCKED")
				if v.UnregisterAllEvents then
					v:UnregisterAllEvents()
				end
			end
		end
	elseif addon == "Stubby" then
		createSwatter()
	end
end

-- Now register for our needed events
frame:SetScript("OnEvent", function(self, event, arg1, arg2)
	if event == "ADDON_ACTION_BLOCKED" or event == "ADDON_ACTION_FORBIDDEN" then
		grabError(L.ADDON_CALL_PROTECTED:format(event, arg1 or "?", arg2 or "?"))
	elseif event == "ADDON_LOADED" then
		addonLoaded(arg1 or "?")
		if not callbacks then setupCallbacks() end
	elseif event == "PLAYER_LOGIN" then
		real_seterrorhandler(grabError)
		if IsAddOnLoaded("Stubby") and type(_G.Swatter) ~= "table" then
			createSwatter()
		end
	end
end)
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("PLAYER_LOGIN")
BugGrabber:RegisterAddonActionEvents()

real_seterrorhandler(grabError)
function seterrorhandler() --[[ noop ]] end

_G.BugGrabber = BugGrabber

-- vim:set ts=4:
